/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.util.*; //for HashMap and LinkedList
import java.io.*;   //for archivation

public class AnatoliaCollection
{
   AreaMap areaMap = null;
   private int currentVnum=0;
   private Map usedVnums = new HashMap(); //<vnum, anatoliaThing>
   private LinkedList someFreeVnums = new LinkedList(); //vnums freed by vnum replacement, or
                                                        //deletion
   protected LinkedList collection = new LinkedList(); //collection of AnatoliaThings

   public int getMaxVnum() {return currentVnum;}

   public AnatoliaCollection(AreaMap areaMap)
   {
     this.areaMap = areaMap;
   }

   public Object[] toArray()
   {
      return collection.toArray();
   }

   public Iterator getIterator() {return collection.iterator();}

   public void establishLinks()
   {
      Iterator it = collection.iterator();
      AnatoliaThing anatoliaThing = null;
      while (it.hasNext())
      {
        anatoliaThing = (AnatoliaThing) it.next();
        anatoliaThing.establishLinks();
      }
  }

  public void clear()
  {
    currentVnum = 0;
    usedVnums.clear();
    someFreeVnums.clear();
    collection.clear();
  }

  public boolean isEmpty()
  {
    return collection.isEmpty();
  }

  public void add(AnatoliaThing anatoliaThing)
  {
    collection.add(anatoliaThing);
  }

  public void remove(AnatoliaThing anatoliaThing)
  {
     Integer vnum = new Integer(anatoliaThing.getVnum());
     usedVnums.remove(vnum);
     if ((vnum.intValue() / 1000) == areaMap.areaNumber)
        someFreeVnums.add(vnum);
     collection.remove(anatoliaThing);
  }

  public void setToFirstFreeVnum(AnatoliaThing anatoliaThing)
  {
      //first time, inits the first vnum of the area map
      if (currentVnum <= this.areaMap.areaNumber*1000)
      {
         currentVnum = this.areaMap.areaNumber*1000;
      }

      if (!someFreeVnums.isEmpty())
      {
         int tempVnum = ((Integer) someFreeVnums.removeFirst()).intValue();
         setVnum(anatoliaThing, tempVnum);
      }
      else
      {
         while (!setVnum(anatoliaThing, ++currentVnum));
      }
   }

   //returns true for success, false for failure
   public boolean setVnum(AnatoliaThing anatoliaThing, int vnum)
   {
      if (vnum == anatoliaThing.getVnum()) return true;
      AnatoliaThing concurrent = getAnatoliaThingOfVnum(vnum);
      if (concurrent == null)
      {
         Integer oldVnum = new Integer(anatoliaThing.getVnum());
         usedVnums.remove(oldVnum);
         if (anatoliaThing.getVnum()/1000 == this.areaMap.areaNumber)
            someFreeVnums.add(oldVnum);
         anatoliaThing.imposeVnum(vnum);
         usedVnums.put(new Integer(vnum), anatoliaThing);
         return true;
      }
      else
       return false;
   }
   public void imposeVnum(AnatoliaThing anatoliaThing, int vnum)
   {
      Integer oldVnum = new Integer(anatoliaThing.getVnum());
      usedVnums.put(new Integer(vnum), anatoliaThing);
   }

   public AnatoliaThing getAnatoliaThingOfVnum(int vnum)
   {
      return (AnatoliaThing) usedVnums.get(new Integer(vnum));
   }

  public void refreshVnums()
  {
     int newCurrentVnum=0;
     Map newUsedVnums = new HashMap(); //<vnum, anatoliaThing>
     LinkedList newSomeFreeVnums = new LinkedList(); //vnums freed by vnum replacement, or deletion
     int maxVnum=0;
     int minVnum=999999;
     Set entrySet = usedVnums.entrySet();
     Iterator it = entrySet.iterator();
     Map.Entry entry = null;
     int vnum = 0;
     AnatoliaThing anatoliaThing = null;
     //refresh usedvnums
     while(it.hasNext())
     {
        entry = (Map.Entry) it.next();
        vnum = ((Integer) entry.getKey()).intValue();
        anatoliaThing = (AnatoliaThing) entry.getValue();
        if (vnum>maxVnum) maxVnum = vnum;
        if (vnum<minVnum) minVnum = vnum;
        newUsedVnums.put(new Integer(vnum), anatoliaThing);
     }
     usedVnums = newUsedVnums;
     currentVnum = maxVnum+1;

     //refresh someFreeVnums
     for(int i=minVnum ; i<maxVnum ; ++i)
     {
       Integer j = new Integer(i);
       if (newUsedVnums.get(j) == null) someFreeVnums.add(j);
     }
     someFreeVnums = newSomeFreeVnums;
     System.gc();
  }

  public void writeObject(ObjectOutputStream output) throws IOException
  {
      output.writeInt(currentVnum);
      output.writeInt(someFreeVnums.size());
      Iterator it = someFreeVnums.listIterator();
      while(it.hasNext())
      {
         output.writeInt(((Integer) it.next()).intValue());
      }
  }

  public void readObject(ObjectInputStream input) throws IOException, ClassNotFoundException
  {
      currentVnum = input.readInt();
      int tempSize = input.readInt();
      someFreeVnums.clear();
      for(int i=0 ; i<tempSize ; ++i)
      {
         someFreeVnums.add(new Integer(input.readInt()));
      }
  }

  public void changeAreaNumber(int oldAreaNumber, int newAreaNumber)
  {
     if ((oldAreaNumber<1) || (oldAreaNumber>99) || (newAreaNumber<1) || (newAreaNumber>99)) return;
     if (currentVnum !=0) currentVnum = newAreaNumber*1000+(currentVnum%1000);
     LinkedList newSomeFreeVnums = new LinkedList();
     Iterator it = someFreeVnums.listIterator();
     int number = 0;
     while (it.hasNext())
     {
        number = ((Integer) it.next()).intValue();
        newSomeFreeVnums.add(new Integer(newAreaNumber*1000+number%1000));
     }
     someFreeVnums = newSomeFreeVnums;

     Map newUsedVnums = new HashMap();
     Set entries = usedVnums.entrySet();
     it = entries.iterator();
     Map.Entry entry = null;
     AnatoliaThing anatoliaThing = null;
     boolean change = true;
     while (it.hasNext())
     {
        entry = (Map.Entry) it.next();
        number = ((Integer) entry.getKey()).intValue();
        anatoliaThing = (AnatoliaThing) entry.getValue();
        change = true;
        try
        {
           AnatoliaRoom room = (AnatoliaRoom) anatoliaThing;
           if (room.isExtern()) change = false;
        }
        catch(Exception e) {}

        if ((change) && (number/1000 == oldAreaNumber))
        {
           anatoliaThing.setVnum(newAreaNumber*1000+anatoliaThing.getVnum()%1000,false);
           newUsedVnums.put(new Integer(anatoliaThing.getVnum()), anatoliaThing);
        }
        else
           newUsedVnums.put(entry.getKey(), anatoliaThing);
     }
     usedVnums = newUsedVnums;
  }

  public AnatoliaObject[] getAllKeyObjects()
  {
     LinkedList list = new LinkedList();
     AnatoliaObject object;
     Set entrySet = usedVnums.entrySet();
     Iterator it = entrySet.iterator();
     Map.Entry entry = null;
     while(it.hasNext())
     {
        entry = (Map.Entry) it.next();
        object = (AnatoliaObject) entry.getValue();
        if (object.type == AnatoliaConsts.OBJECT_TYPE_KEY) list.add(object);
     }
     AnatoliaObject[] tab = new AnatoliaObject[list.size()];
     it = list.iterator();
     int i=0;
     while(it.hasNext())
     {
        tab[i++] = (AnatoliaObject) it.next();
     }
     return tab;
  }

}