/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.io.*;
import java.util.*; //pour StringTokenizer

public class AnatoliaObjectsReader extends AnatoliaAreReader
{
  public AnatoliaObjectsReader(AreaMap areaMap)
  {
    super(areaMap);
  }

  public void read(BufferedReader input) throws IOException
  {
     String line = super.trim(input);
     while(!line.startsWith("#0"))
     {
       AnatoliaObject object = new AnatoliaObject(areaMap.theObjects);

       //vnum
       //object.setVnum(MyInteger.parseInt(line.substring(1)));
       object.imposeVnum(MyInteger.parseInt(line.substring(1)));

       //name list
       line = super.trim(input);
       object.name = line.substring(0,Math.max(0,line.length()-1));

       //short desc
       line = super.trim(input);
       object.shortDescription = line.substring(0,Math.max(0,line.length()-1));

       //long desc
       line = "";
       do
       {
         line = line + "\n" + super.trim(input);
       } while (!line.endsWith("~"));
       object.longDescription = line.substring(0,Math.max(0,line.length()-1)).trim();

       //material
       line = super.trim(input).toLowerCase();
       while (line.indexOf("~")<0) line = super.trim(input);
       object.material = line.substring(0,Math.max(0,line.length()-1));

       //type extra:flags wear:flags
       line = super.trim(input);
       StringTokenizer tokens = new StringTokenizer(line);
       String type = tokens.nextToken().toLowerCase();
       try
       {
         int candidate = Integer.parseInt(type); //check if it's a number
         object.type = AreaBuilderConsts.findInTab(AnatoliaConsts.getObjectTypeCodes(), type);
       }
       catch(Exception e)
       {
         object.type = AreaBuilderConsts.findInTab(AnatoliaConsts.getObjectTypeTexts(), type);
       }
       if (tokens.hasMoreTokens()) object.extraFlags = AreaBuilderConsts.interpretFlags(tokens.nextToken());
       if (tokens.hasMoreTokens()) object.wearFlags = AreaBuilderConsts.interpretFlags(tokens.nextToken());

       //v0 v1 v2 v3 v4
       line = super.trim(input);
       tokens = new StringTokenizer(line);
       object.v0 = "0";
       object.v1 = "0";
       object.v2 = "0";
       object.v3 = "0";
       object.v4 = "0";
       if (tokens.hasMoreTokens()) object.v0 = tokens.nextToken(" \t\n\r\f");
       if (object.v0.startsWith("'") && !object.v0.endsWith("'"))
       {
         object.v0 = object.v0+tokens.nextToken("'")+"'";
       }
       if (tokens.hasMoreTokens()) object.v1 = tokens.nextToken(" \t\n\r\f");
       if (object.v1.startsWith("'") && !object.v1.endsWith("'"))
       {
         object.v1 = object.v1+tokens.nextToken("'")+"'";
       }
       if (tokens.hasMoreTokens()) object.v2 = tokens.nextToken(" \t\n\r\f");
       if (object.v2.startsWith("'") && !object.v2.endsWith("'"))
       {
         object.v2 = object.v2+tokens.nextToken("'")+"'";
       }
       if (tokens.hasMoreTokens()) object.v3 = tokens.nextToken(" \t\n\r\f");
       if (object.v3.startsWith("'") && !object.v3.endsWith("'"))
       {
         object.v3 = object.v3+tokens.nextToken("'")+"'";
       }
       if (tokens.hasMoreTokens()) object.v4 = tokens.nextToken(" \t\n\r\f");
       if (object.v4.startsWith("'") && !object.v4.endsWith("'"))
       {
         object.v4 = object.v3+tokens.nextToken("'")+"'";
       }

       //level weight cost cond
       line = super.trim(input);
       tokens = new StringTokenizer(line);
       if (tokens.hasMoreTokens()) object.level = (byte) MyInteger.parseInt(tokens.nextToken());
       if (tokens.hasMoreTokens()) object.weight = MyInteger.parseInt(tokens.nextToken());
       if (tokens.hasMoreTokens()) object.cost = MyInteger.parseInt(tokens.nextToken());
       if (tokens.hasMoreTokens()) object.cond = tokens.nextToken();
       //optional: A or F or E
       line = super.trim(input);
       while(line.startsWith("A") || line.startsWith("F") || line.startsWith("E"))
       {
         if (line.startsWith("A")) //applies
         {
           if (line.equals("A"))
           {
             line = super.trim(input);
             tokens = new StringTokenizer(line);
           }
           else
           {
             tokens = new StringTokenizer(line);
             tokens.nextToken(); //read "A"
           }
           String locationCode = tokens.nextToken();
           int modifier = MyInteger.parseInt(tokens.nextToken());
           int locationIndex = AreaBuilderConsts.findInTab(AnatoliaConsts.getApplyCodes(), locationCode);
           if (locationIndex>0)
             object.localAffects[locationIndex] += modifier;
         }
         if (line.startsWith("E")) //extra description
         {
           String keyword = super.trim(input);
           keyword = keyword.substring(0,Math.max(0,keyword.length()-1));
           line = "";
           do
           {
             line = line + "\n" + super.trim(input);
           } while (!line.endsWith("~"));
           line = line.substring(0,Math.max(0,line.length()-1)).trim();
           String description = new String(line);
           object.extraDescriptions.add(new Pair(keyword, description));
         }
         if (line.startsWith("F")) //flags F AIRV 0 0 A
         {
           if (line.equals("F"))
           {
             line = super.trim(input);
             tokens = new StringTokenizer(line);
           }
           else
           {
             tokens = new StringTokenizer(line);
             tokens.nextToken(); //read "F"
           }
           type = tokens.nextToken();
           String locationCode = tokens.nextToken();
           int modifier = MyInteger.parseInt(tokens.nextToken());
           String bits = tokens.nextToken();
           int locationIndex = AreaBuilderConsts.findInTab(AnatoliaConsts.getApplyCodes(), locationCode);
           if (locationIndex>0)
             object.localAffects[locationIndex] += modifier;

           if (type.equals("A")) //apply
              object.generalAffects += bits;
           else if (type.equals("I")) //immune
              object.immFlags += bits;
           else if (type.equals("R")) //resistance
              object.resFlags += bits;
           else if (type.equals("V")) //vulnerable
              object.vulnFlags += bits;
           else if (type.equals("D")) //detection
              object.detectFlags += bits;
         }
         line = super.trim(input);
       } // end optional A F E
       areaMap.theObjects.add(object);
    } // end while obj
  } //end function
} //end class