/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.util.Stack;

public abstract class UndoableAction
{
  public static final boolean ALLOW_UNDO_YES = true;
  public static final boolean ALLOW_UNDO_NO = false;
  public static final boolean AUTOMATIC_ACTION = true;
  public static final boolean NOT_AUTOMATIC_ACTION = false;

  //an action may apply its own sub-actions, to be undoed too...
  protected UndoableActionStack actionStack = null;

  //if an action is automatic, il will be undoed/redoed automatically
  protected boolean automatic = false;
  boolean isAutomatic() {return automatic;}

  protected UndoableAction(boolean automatic)
  {
     this.automatic = automatic;
     actionStack = new UndoableActionStack();
  }
  protected UndoableAction()
  {
     this.automatic = NOT_AUTOMATIC_ACTION;
     actionStack = new UndoableActionStack();
  }

  abstract public void doAction();
  abstract public void undoAction();
  abstract public void redoAction();
}