/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.awt.*;
import java.util.*;
import java.awt.geom.*; //dor Line2D, Point2D,...
import java.io.*; //for Archivation

public class Wire extends LinkingAndSelectableNodeObject implements Serializable
{
  private LinkedList pointList = new LinkedList();
  private Room fromRoom = null;
  private long fromRoomID;
  private byte fromDoor = Room.DOORNONE;
  private Room toRoom = null;
  private long toRoomID;
  private byte toDoor = Room.DOORNONE;
  private float scale;
  private int indexOfSelectedSegment = -1;
  private boolean allWireSelected = false;

  public Room getFromRoom()           {return fromRoom;}
  public Room getToRoom()             {return   toRoom;}
  public byte getFromDoor()           {return fromDoor;}
  public byte getToDoor()             {return   toDoor;}
  public boolean getAllWireSelected() {return allWireSelected;}

  public Wire() {}

  public Object clone()
  {
     Wire wire = new Wire();
     Iterator it = this.pointList.listIterator();
     while (it.hasNext())
     {
        wire.pointList.add(((Point) it.next()).clone());
     }
     wire.fromRoom   = this.fromRoom;
     wire.fromRoomID = this.fromRoomID;
     wire.fromDoor   = this.fromDoor;
     wire.toRoom     = this.toRoom;
     wire.toRoomID   = this.toRoomID;
     wire.toDoor     = this.toDoor;
     wire.scale      = this.scale;
     wire.indexOfSelectedSegment = this.indexOfSelectedSegment;
     wire.allWireSelected        = this.allWireSelected;
     return wire;
  }

  public LinkedList getPointList() {return pointList;}
  public void setPointList(LinkedList pointList) {this.pointList = (LinkedList) pointList;}

  public void reconnect(Room fromRoom, byte fromDoor, Room toRoom, byte toDoor)
  {
     Room.disconnect(this.fromRoom, this.fromDoor, this.toRoom, this.toDoor, this);
     this.fromRoom = fromRoom;
     this.fromDoor = fromDoor;
     this.toRoom = toRoom;
     this.toDoor = toDoor;
     Room.connect(this.fromRoom, this.fromDoor, this.toRoom, this.toDoor, this);
     if ((fromRoom == toRoom) && (pointList.size() == 0))
       specialCase(fromRoom, fromDoor, toDoor);
  }

  public void specialCase(Room room, byte fromDoor, byte toDoor)
  {
    switch(fromDoor)
    {
      case Room.DOORNORTH : if (toDoor == Room.DOORSOUTH)
                       {
                         pointList.add(new Point(room.getXOnScreen()-10, room.getYOnScreen()-10));
                         pointList.add(new Point(room.getXOnScreen()-10, room.getYOnScreen()+room.getSizeOnScreen()+10));
                       }
                       break;
      case Room.DOOREAST : if (toDoor == Room.DOORWEST)
                       {
                         pointList.add(new Point(room.getXOnScreen()+room.getSizeOnScreen()+10, room.getYOnScreen()-10));
                         pointList.add(new Point(room.getXOnScreen()-10, room.getYOnScreen()-10));
                       }
                       break;
      case Room.DOORWEST : if (toDoor == Room.DOOREAST)
                       {
                         pointList.add(new Point(room.getXOnScreen()-10, room.getYOnScreen()-10));
                         pointList.add(new Point(room.getXOnScreen()+room.getSizeOnScreen()+10, room.getYOnScreen()-10));
                       }
                       break;
      case Room.DOORSOUTH : if (toDoor == Room.DOORNORTH)
                       {
                         pointList.add(new Point(room.getXOnScreen()-10, room.getYOnScreen()+room.getSizeOnScreen()+10));
                         pointList.add(new Point(room.getXOnScreen()-10, room.getYOnScreen()-10));
                       }
                       break;
      case Room.DOORUP : if (toDoor == Room.DOORDOWN)
                       {
                         pointList.add(new Point(room.getXOnScreen()-10, room.getYOnScreen()+room.getSizeOnScreen()+10));
                       }
                       break;
      case Room.DOORDOWN : if (toDoor == Room.DOORUP)
                       {
                         pointList.add(new Point(room.getXOnScreen()+room.getSizeOnScreen()+10, room.getYOnScreen()-10));
                       }
                       break;
      default : break;
    }
  }

  public void createConnect(Room fromRoom, byte fromDoor, Room toRoom, byte toDoor)
  {
     this.fromRoom = fromRoom;
     this.fromDoor = fromDoor;
     this.toRoom = toRoom;
     this.toDoor = toDoor;
     if (fromDoor != Room.DOORNONE)
     {
        fromRoom.connectingToRooms[fromDoor] = new Pair(toRoom, this);
        toRoom.mayComeFromRooms.add(new Pair(fromRoom, new Byte(fromDoor)));
     }
     if (toDoor != Room.DOORNONE)
     {
        toRoom.connectingToRooms[toDoor] = new Pair(fromRoom, this);
        fromRoom.mayComeFromRooms.add(new Pair(toRoom, new Byte(toDoor)));
     }

     if ((fromRoom == toRoom) && (pointList.size() == 0)) specialCase(fromRoom, fromDoor, toDoor);
  }

  public void setAllWireSelected(boolean state)
  {
     allWireSelected = state;
  }

  public void unselect()
  {
     allWireSelected = false;
     indexOfSelectedSegment = -1;
  }

  public Point getLastPoint()
  {
     if (pointList.isEmpty())
        return fromRoom.getDoorPoint(fromDoor);
     else
        return (Point) pointList.getLast();
  }

  public int size() {return pointList.size();}
  public void removeLastPoint()
  {
     if (pointList.isEmpty() == false) pointList.removeLast();
  }

  public void clear()
  {
     pointList.clear();
     fromRoom = toRoom = null;
     fromDoor = toDoor = Room.DOORNONE;
     scale = 1.f;
     indexOfSelectedSegment = -1;
     setAllWireSelected(false);
  }

  //return true if ok, false if impossible
  public boolean open(int xMouse, int yMouse, Room firstRoom, float scale)
  {
     clear();
     this.scale = scale;
     fromRoom = firstRoom;
     fromDoor = fromRoom.getDoor(xMouse, yMouse);
     return fromRoom.isDoorFree(fromDoor);
  }

  //return true if ok, false if impossible
  public boolean close(int xMouse, int yMouse, Room lastRoom)
  {
     Room candidateToRoom = lastRoom;
     toDoor = candidateToRoom.getDoor(xMouse, yMouse);
     if ((fromRoom == candidateToRoom) && (fromDoor == toDoor)) return false;
     if ((fromDoor == Room.DOORNONE) && (toDoor == Room.DOORNONE)) return false;
     if (candidateToRoom.isDoorFree(toDoor))
     {
       if ((fromRoom == candidateToRoom) && (pointList.size() == 0))
       {
         specialCase(fromRoom, fromDoor, toDoor);
       }
       toRoom = candidateToRoom;
       return true;
     }
     else
     {
       return false;
     }
  }

  public void add(Point point)
  {
     pointList.add(new Point(point));
  }

  public void reScale(float newScale)
  {
     Iterator it = pointList.iterator();
     Point point = null;
     float factor = newScale/scale;
     while (it.hasNext())
     {
        point = (Point) it.next();
        point.x = (int) (point.x*factor);
        point.y = (int) (point.y*factor);
     }
     scale = newScale;
  }

  public boolean isGoingThrough(Point point)
    {return isGoingThrough(point, 5);}

  public boolean isGoingThrough(Point point, int tolerance)
  {
     Line2D.Float line2D = new Line2D.Float();
     Point2D point2D = (Point2D) point;
     Point2D point1 = (Point2D) fromRoom.getDoorPoint(fromDoor);
     Point2D point2 = null;

     indexOfSelectedSegment = -1;
     Iterator it = pointList.iterator();
     while (it.hasNext())//FromRoom->point
     {
         ++indexOfSelectedSegment;
         point2 = (Point2D) it.next();
         line2D.setLine(point1, point2);
         if (line2D.ptSegDist(point2D) <= tolerance)
            return true;
         point1 = point2;
     }
     ++indexOfSelectedSegment;
     point2 = (Point2D) toRoom.getDoorPoint(toDoor);
     line2D.setLine(point1, point2);
     if (line2D.ptSegDist(point2D) <= tolerance)
         return true;

     indexOfSelectedSegment=-1;
     return false;
  }

  public boolean removeSelectedSegment() //true if wire must be deleted
  {
     if ((indexOfSelectedSegment < 0) || (indexOfSelectedSegment > pointList.size())) return false;
     if (pointList.isEmpty()) return true;
     if (indexOfSelectedSegment == pointList.size()) --indexOfSelectedSegment;

     pointList.remove(indexOfSelectedSegment);
     return false;
  }

  public void translate(int deltaXOnScreen, int deltaYOnScreen)
  {
    Iterator it = pointList.iterator();
    Point point = null;
    while (it.hasNext())
    {
      point = (Point) it.next();
      point.translate(deltaXOnScreen, deltaYOnScreen);
    }
  }

  public void paint(Graphics g)
  {
     boolean okFrom = (fromRoom == null) ||
                      (fromRoom.areaMap.selectedLayers.contains(new Integer(fromRoom.layer)));
     boolean okTo   = (toRoom == null) ||
                      (toRoom.areaMap.selectedLayers.contains(new Integer(toRoom.layer)));
     Color wireColor = Color.black;
     if (!okFrom && !okTo)
     {
        wireColor = Color.magenta;
     }

     Point point1 = fromRoom.getDoorPoint(fromDoor);
     Point point2 = null;

     if (allWireSelected)
       g.setColor(Color.red);
     else
       g.setColor(wireColor);

     Iterator it = pointList.iterator();
     int currentIndexOfSegment = -1;

     while (it.hasNext())
     {
        point2 = (Point) it.next();
        if ((++currentIndexOfSegment) == indexOfSelectedSegment) g.setColor(Color.red);
        g.drawLine(point1.x, point1.y, point2.x, point2.y);
        if (allWireSelected == false) g.setColor(wireColor);
        point1 = point2;
     }

     if (toRoom != null)
     {
        point2 = toRoom.getDoorPoint(toDoor);
        if ((++currentIndexOfSegment) == indexOfSelectedSegment)
        {
         g.setColor(Color.red);
        }
        g.drawLine(point1.x, point1.y, point2.x, point2.y);
     }
     g.setColor(Color.black);
  }

  public void writeObject(ObjectOutputStream output) throws IOException
  {
     output.writeLong(super.getObjectID());
     output.writeLong(fromRoom.getObjectID());
     output.writeByte(fromDoor);
     output.writeLong(toRoom.getObjectID());
     output.writeByte(toDoor);
     output.writeFloat(scale);
     output.writeInt(indexOfSelectedSegment);
     output.writeInt(pointList.size());
     Iterator it = pointList.iterator();
     Point point = null;
     while(it.hasNext())
     {
        point = (Point) it.next();
        output.writeInt(point.x);
        output.writeInt(point.y);
     }
  }

 public void readObject(ObjectInputStream input) throws IOException, ClassNotFoundException
 {
    super.setObjectID(input.readLong());

    fromRoomID = input.readLong();
    fromDoor = input.readByte();

    toRoomID = input.readLong();
    toDoor = input.readByte();

    scale = input.readFloat();

    indexOfSelectedSegment = input.readInt();
    int sizeOfPointList    = input.readInt();
    for(int i=0; i<sizeOfPointList; ++i)
    {
       pointList.add(new Point(input.readInt(), input.readInt()));
    }
 }

 public void establishLinks()
 {
    fromRoom = (Room) super.lookForObjectOfID(fromRoomID);
    toRoom   = (Room) super.lookForObjectOfID(toRoomID);
 }
}